#!/bin/env php
<?php

require_once __DIR__.'/../inc_functions.php';
require_once __DIR__.'/../inc_vars.php';

$AMCLI??=__DIR__.'/../src/amcli.php';
$AMCLIBIN??=__DIR__.'/../built_packages/amcli';

// --------------------------------------------------------------------

/**
 * Get array of files with tests
 * @return array
 */
function getTests(): array
{
    $aReturn=[];
    foreach(glob(__DIR__."/*.php") as $sFile){
        if(basename($sFile)!=basename(__FILE__)){
            $aReturn[]=$sFile;
        }
    }
    return $aReturn;
}

// --------------------------------------------------------------------

if ($argc > 1) {
    parse_str(implode('&', array_slice($argv, 1)), $ARGS);
}

if(isset($ARGS['-h']) || isset($ARGS['--help'])){
    echo "
This ist the test suite for amcli calls.
It executes all scripts in this directory. They perform checks for metadata, 
help, all checks and ini features.

 ✨ \e[1mSYNTAX:\e[0m

    ./00_start.php [-b]

 🔷 \e[1mOPTIONS:\e[0m

    -h, --help           Show this help
    -b, --bin [=<FILE>]  Set amcli binary. If a FILE is given then use that one.
                         Default is the php script
                         $AMCLI

                         Default of -b without file is
                         $AMCLIBIN
    -l, --list           list filenames of tests

";
    exit(0);
}

if(isset($ARGS['-l']) || isset($ARGS['--list'])){
    foreach(getTests() as $sFile){
        echo basename($sFile)."\n";
    }
    exit(0);
}

if(isset($ARGS['-b']) || isset($ARGS['--bin'])){
    $AMCLI=($ARGS['--bin'] ?? $ARGS['-b']) ?: $AMCLIBIN;
    if(!file_exists($AMCLI)){
        echo "ERROR: [$AMCLI] not found\n";
        exit(1);
    }
}
echo "Using $AMCLI for testing\n\n";

// _chdir(__DIR__);

echo "\n\nSTART TESTS\n\n";
foreach(getTests() as $sFile){
    _h1("$sFile");
    include $sFile;
}

_h1("DONE");

// --------------------------------------------------------------------
